/*
 *   ccache_list.h -- definitions for CS 2270 linked list package
 *   
 *   Mark Peterson  9/91
 *
 *  ccache_list.h,v 1.8 1995/09/05 19:09:30 hardy Exp
 *
 *  ----------------------------------------------------------------------
 *  Copyright (c) 1994, 1995.  All rights reserved.
 *  
 *    The Harvest software was developed by the Internet Research Task
 *    Force Research Group on Resource Discovery (IRTF-RD):
 *  
 *          Mic Bowman of Transarc Corporation.
 *          Peter Danzig of the University of Southern California.
 *          Darren R. Hardy of the University of Colorado at Boulder.
 *          Udi Manber of the University of Arizona.
 *          Michael F. Schwartz of the University of Colorado at Boulder.
 *          Duane Wessels of the University of Colorado at Boulder.
 *  
 *    This copyright notice applies to software in the Harvest
 *    ``src/'' directory only.  Users should consult the individual
 *    copyright notices in the ``components/'' subdirectories for
 *    copyright information about other software bundled with the
 *    Harvest source code distribution.
 *  
 *  TERMS OF USE
 *    
 *    The Harvest software may be used and re-distributed without
 *    charge, provided that the software origin and research team are
 *    cited in any use of the system.  Most commonly this is
 *    accomplished by including a link to the Harvest Home Page
 *    (http://harvest.cs.colorado.edu/) from the query page of any
 *    Broker you deploy, as well as in the query result pages.  These
 *    links are generated automatically by the standard Broker
 *    software distribution.
 *    
 *    The Harvest software is provided ``as is'', without express or
 *    implied warranty, and with no support nor obligation to assist
 *    in its use, correction, modification or enhancement.  We assume
 *    no liability with respect to the infringement of copyrights,
 *    trade secrets, or any patents, and are not responsible for
 *    consequential damages.  Proper use of the Harvest software is
 *    entirely the responsibility of the user.
 *  
 *  DERIVATIVE WORKS
 *  
 *    Users may make derivative works from the Harvest software, subject 
 *    to the following constraints:
 *  
 *      - You must include the above copyright notice and these 
 *        accompanying paragraphs in all forms of derivative works, 
 *        and any documentation and other materials related to such 
 *        distribution and use acknowledge that the software was 
 *        developed at the above institutions.
 *  
 *      - You must notify IRTF-RD regarding your distribution of 
 *        the derivative work.
 *  
 *      - You must clearly notify users that your are distributing 
 *        a modified version and not the original Harvest software.
 *  
 *      - Any derivative product is also subject to these copyright 
 *        and use restrictions.
 *  
 *    Note that the Harvest software is NOT in the public domain.  We
 *    retain copyright, as specified above.
 *  
 *  HISTORY OF FREE SOFTWARE STATUS
 *  
 *    Originally we required sites to license the software in cases
 *    where they were going to build commercial products/services
 *    around Harvest.  In June 1995 we changed this policy.  We now
 *    allow people to use the core Harvest software (the code found in
 *    the Harvest ``src/'' directory) for free.  We made this change
 *    in the interest of encouraging the widest possible deployment of
 *    the technology.  The Harvest software is really a reference
 *    implementation of a set of protocols and formats, some of which
 *    we intend to standardize.  We encourage commercial
 *    re-implementations of code complying to this set of standards.  
 *  
 *  
 */
#ifndef _CCACHE_LIST_H_
#define _CCACHE_LIST_H_

typedef struct list_node {	/*list node type */
	struct list_node *next;	/*points to the following node */
	struct list_node *previous;	/*points to the previous node */
	Datum *data;		/*stores data record in list */
} List_Node;


typedef struct {		/*list header node */
	List_Node *first;	/*points to the first node */
	List_Node *last;	/*points to the last node */
	unsigned int count;
	/*keeps count of the number of nodes in list */
	int (*compare) ();	/*A compare function */
} Linked_List;


/*
   **The list toolkit functions:
 */

Linked_List *list_create();	/*initialize list header block */
void list_destroy();		/*destroy list header block */
List_Node *list_insert();	/*insert a new node in the list */
Datum *list_delete();		/*delete a node from the list */
List_Node *list_find();		/*find a node in the list */
Boolean list_apply();		/*apply a function to each node in the list */

/*Built in Macros */
#define list_first(head) ((head)->first)	/*find the first node in the list */
#define list_next(node) ((node)->next)	/*find the next node in the list */
#define list_last(head) ((head)->last)	/*find the last node in the list */
#define list_previous(node) ((node)->previous)
/*find the previous node in the list */

#define list_getdata(node) ((node)->data)	/*get the data from the list */
void list_putdata();		/*modify the data in the node in the list */
#define list_length(head) ((head)->count)	/*find the length of the list */

#endif
